﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Chop45
{
    public partial class Form1 : Form
    {
        string startCurDir;

        // コンストラクタ
        public Form1()
        {
            InitializeComponent();
        }

        // フォームのロード時
        private void Form1_Load(object sender, EventArgs e)
        {
            // 自分がいるディレクトリはタイムスタンプを変更できないので安全な場所へ
            startCurDir = Directory.GetCurrentDirectory();
            Directory.SetCurrentDirectory("C:\\");
        }

        // フォームの初回表示時
        private void Form1_Shown(object sender, EventArgs e)
        {
            // コマンドライン引数が指定されていたら、それで実行
            if (Environment.GetCommandLineArgs().Length > 1) {
                string targetPath = Environment.GetCommandLineArgs()[1];

                // フルパスに変換
                targetPath = startCurDir + "\\" + targetPath;
                if (targetPath != string.Empty) {

                    // 存在チェック
                    if (Directory.Exists(targetPath)) {
                        StartSearch(targetPath);
                        MessageBox.Show("Succeeded!\n" + targetPath);

                        this.Close();
                    }
                    else {
                        MessageBox.Show("Specified path does not exist.\n" + targetPath);
                        this.Close();
                    }
                }
            }
        }

        // ドラッグが来た
        private void textPath_DragEnter(object sender, DragEventArgs e)
        {
            // 存在するディレクトリなら「コピー」アイコン
            if (Directory.Exists(((string[])e.Data.GetData(DataFormats.FileDrop))[0])) {
                e.Effect = DragDropEffects.Copy;
            }
        }

        // ディレクトリのドロップ
        private void textPath_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop)) {
                textPath.Text = ((string[])e.Data.GetData(DataFormats.FileDrop))[0];
            }
        }

        // 参照ボタン
        private void btnBrowse_Click(object sender, EventArgs e)
        {
            // ディレクトリ選択ダイアログを表示
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            fbd.SelectedPath = startCurDir;
            if (fbd.ShowDialog() == DialogResult.OK) {
                textPath.Text = fbd.SelectedPath;
            }
        }

        // ディレクトリ名テキストボックス更新時
        private void textPath_TextChanged(object sender, EventArgs e)
        {
            // 存在するディレクトリなら実行ボタンを有効にする
            if (Directory.Exists(textPath.Text))
                btnChop.Enabled = true;
            else
                btnChop.Enabled = false;
        }

        // 実行ボタン
        private void btnChop_Click(object sender, EventArgs e)
        {
            // 未入力なら無視
            if (textPath.Text == string.Empty) {
                return;
            }
            StartSearch(textPath.Text);
        }

        // 検索開始
        public bool StartSearch(string startPath)
        {
            lvLog.Items.Clear();

            Search(startPath, "*.*");

            // リストビューの幅を調整し直す
            columnHeader1.Width = -1;
            columnHeader2.Width = -1;

            MessageBox.Show("Finished!\n" + textPath.Text);

            return true;
        }

        // ディレクトリの中の検索ループ
        void Search(string dir, string file)
        {
            if (dir.Normalize() != dir)
            {
                string dest;
                do
                {
                    dest = dir + (new Random()).Next(10000).ToString();
                } while (Directory.Exists(dest));
                Directory.Move(dir, dest);
                Directory.Move(dest, dir.Normalize());
                ListViewItem lvItem = lvLog.Items.Add(dir);
                lvItem.SubItems.Add(dir.Normalize());
                dir = dir.Normalize();
            }

            // ディレクトリ列挙
            string[] dirs = Directory.GetDirectories(dir);
            foreach (string s in dirs) {

                // 再帰呼び出し
                Search(s, file);
            }

            // ファイルを列挙して最新タイムスタンプを取得
            string[] files = Directory.GetFiles(dir, file);
            foreach (string s in files) {
                try
                {
                    if (s != s.Normalize())
                    {
                        File.Move(s, s.Normalize());
                        // ログ出力
                        ListViewItem lvItem = lvLog.Items.Add(s);
                        lvItem.SubItems.Add(s.Normalize());
                    }
                }
                catch (System.Exception e)
                {
                    MessageBox.Show("GetLastWriteTime failed in Search :\n" + e.ToString());
                }
            }
        }
    }
}
